@@ -34,7 +34,7 @@ ILLEGAL_TABLE_TYPE_WITH_VALUE = """\
 """
 
 
-def _read_item(buf, offset=0, unpack_from=unpack_from, ftype_t=ftype_t):
+def _read_item(buf, offset):
     ftype = ftype_t(buf[offset]) if ftype_t else buf[offset]
     offset += 1
 
@@ -42,7 +42,11 @@ def _read_item(buf, offset=0, unpack_from=unpack_from, ftype_t=ftype_t):
     if ftype == 'S':
         slen, = unpack_from('>I', buf, offset)
         offset += 4
-        val = pstr_t(buf[offset:offset + slen])
+        try:
+            val = pstr_t(buf[offset:offset + slen])
+        except UnicodeDecodeError:
+            val = buf[offset:offset + slen]
+
         offset += slen
     # 's': short string
     elif ftype == 's':
@@ -144,9 +148,7 @@ def _read_item(buf, offset=0, unpack_from=unpack_from, ftype_t=ftype_t):
     return val, offset
 
 
-def loads(format, buf, offset=0,
-          ord=ord, unpack_from=unpack_from,
-          _read_item=_read_item, pstr_t=pstr_t):
+def loads(format, buf, offset):
     """Deserialize amqp format.
 
     bit = b
@@ -171,11 +173,11 @@ def loads(format, buf, offset=0,
         if p == 'b':
             if not bitcount:
                 bits = ord(buf[offset:offset + 1])
-            bitcount = 8
+                offset += 1
+                bitcount = 8
             val = (bits & 1) == 1
             bits >>= 1
             bitcount -= 1
-            offset += 1
         elif p == 'o':
             bitcount = bits = 0
             val, = unpack_from('>B', buf, offset)
@@ -245,7 +247,7 @@ def loads(format, buf, offset=0,
     return values, offset
 
 
-def _flushbits(bits, write, pack=pack):
+def _flushbits(bits, write):
     if bits:
         write(pack('B' * len(bits), *bits))
         bits[:] = []
@@ -325,7 +327,7 @@ def dumps(format, values):
     return out.getvalue()
 
 
-def _write_table(d, write, bits, pack=pack):
+def _write_table(d, write, bits):
     out = BytesIO()
     twrite = out.write
     for k, v in items(d):
@@ -343,7 +345,7 @@ def _write_table(d, write, bits, pack=pack):
     write(table_data)
 
 
-def _write_array(l, write, bits, pack=pack):
+def _write_array(l, write, bits):
     out = BytesIO()
     awrite = out.write
     for v in l:
@@ -357,11 +359,7 @@ def _write_array(l, write, bits, pack=pack):
     write(array_data)
 
 
-def _write_item(v, write, bits, pack=pack,
-                string_t=string_t, bytes=bytes, string=string, bool=bool,
-                float=float, int_types=int_types, Decimal=Decimal,
-                datetime=datetime, dict=dict, list=list, tuple=tuple,
-                None_t=None):
+def _write_item(v, write, bits):
     if isinstance(v, (string_t, bytes)):
         if isinstance(v, string):
             v = v.encode('utf-8', 'surrogatepass')
@@ -393,14 +391,13 @@ def _write_item(v, write, bits, pack=pack,
     elif isinstance(v, (list, tuple)):
         write(b'A')
         _write_array(v, write, bits)
-    elif v is None_t:
+    elif v is None:
         write(b'V')
     else:
         raise ValueError()
 
 
-def decode_properties_basic(buf, offset=0,
-                            unpack_from=unpack_from, pstr_t=pstr_t):
+def decode_properties_basic(buf, offset):
     """Decode basic properties."""
     properties = {}
 
@@ -507,8 +504,7 @@ class GenericContent(object):
             return self.properties[name]
         raise AttributeError(name)
 
-    def _load_properties(self, class_id, buf, offset=0,
-                         classes=PROPERTY_CLASSES, unpack_from=unpack_from):
+    def _load_properties(self, class_id, buf, offset):
         """Load AMQP properties.
 
         Given the raw bytes containing the property-flags and property-list
@@ -516,7 +512,7 @@ class GenericContent(object):
         stored in this object as an attribute named 'properties'.
         """
         # Read 16-bit shorts until we get one with a low bit set to zero
-        props, offset = classes[class_id](buf, offset)
+        props, offset = PROPERTY_CLASSES[class_id](buf, offset)
         self.properties = props
         return offset
 
